from os.path import join
import os
import unittest
import inspect
import shutil
import tempfile

import mock

from binstar_client.utils import config


class Test(unittest.TestCase):
    def test_signature(self):
        from binstar_client.utils import get_config, set_config

        # make sure that our exported methods have the same interface
        spec = inspect.getargspec(get_config)
        self.assertEqual(spec.args, ['user', 'site', 'remote_site'])
        self.assertEqual(spec.defaults, (True, True, None))

        spec = inspect.getargspec(set_config)
        self.assertEqual(spec.args, ['data', 'user'])
        self.assertEqual(spec.defaults, (True,))

    def test_merge(self):
        tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmpdir)

        system_dir = join(tmpdir, 'system')
        user_dir = join(tmpdir, 'user')
        os.mkdir(system_dir)
        os.mkdir(user_dir)

        with open(join(system_dir, 'config.yaml'), 'wb') as fd:
            fd.write(b'''
ssl_verify: false
sites:
    develop:
        url: http://develop.anaconda.org
            ''')

        with open(join(user_dir, 'config.yaml'), 'wb') as fd:
            fd.write(b'''
ssl_verify: true
sites:
    develop:
        ssl_verify: false
            ''')


        with mock.patch('binstar_client.utils.config.SEARCH_PATH', [system_dir, user_dir]), \
                mock.patch('binstar_client.utils.config.DEFAULT_CONFIG', {}):
            cfg = config.get_config()

            self.assertEqual(cfg, {
                'ssl_verify': True,
                'sites': {
                    'develop': {
                        'url': 'http://develop.anaconda.org',
                        'ssl_verify': False,
                    }
                }
            })






from binstar_client.errors import UserError


class PackageSpec(object):
    def __init__(self, user, package=None, version=None, basename=None, attrs=None, spec_str=None):
        self._user = user
        self._package = package
        self._version = version
        self._basename = basename
        self.attrs = attrs
        if spec_str:
            self.spec_str = spec_str
        else:
            spec_str = str(user)
            if package:
                spec_str = '%s/%s' % (spec_str, package)
            if version:
                spec_str = '%s/%s' % (spec_str, version)
            if basename:
                spec_str = '%s/%s' % (spec_str, basename)
            self.spec_str = spec_str

    def __str__(self):
        return self.spec_str

    def __repr__(self):
        return '<PackageSpec %r>' % (self.spec_str)

    @property
    def user(self):
        if self._user is None:
            raise UserError('user not given (got %r expected <username> )' % (self.spec_str,))
        return self._user

    @property
    def name(self):
        if self._package is None:
            raise UserError('package not given in spec (got %r expected <username>/<package> )' % (self.spec_str,))
        return self._package

    @property
    def package(self):
        if self._package is None:
            raise UserError('package not given in spec (got %r expected <username>/<package> )' % (self.spec_str,))
        return self._package

    @property
    def version(self):
        if self._version is None:
            raise UserError('version not given in spec (got %r expected <username>/<package>/<version> )' % (self.spec_str,))
        return self._version

    @property
    def basename(self):
        if self._basename is None:
            raise UserError('basename not given in spec (got %r expected <username>/<package>/<version>/<filename> )' % (self.spec_str,))
        return self._basename


def package_specs(spec):
    user = spec
    package = None
    attrs = {}
    if '/' in user:
        user, package = user.split('/', 1)
    if '/' in package:
        raise TypeError('invalid package spec')

    return PackageSpec(user, package, None, None, attrs, spec)


def parse_specs(spec):
    user = spec
    package = version = basename = None
    attrs = {}
    if '/' in user:
        user, package = user.split('/', 1)
    if package and '/' in package:
        package, version = package.split('/', 1)

    if version and '/' in version:
        version, basename = version.split('/', 1)

    if basename and '?' in basename:
        basename, qsl = basename.rsplit('?', 1)
        attrs = dict(urlparse.parse_qsl(qsl))

    return PackageSpec(user, package, version, basename, attrs, spec)


class GroupSpec(object):
    def __init__(self, org, group_name=None, member=None, spec_str=None):
        self._org = org
        self._group_name = group_name
        self._member = member

        if not spec_str:
            spec_str = str(org)
            if group_name:
                spec_str = '%s/%s' % (spec_str, group_name)
            if member:
                spec_str = '%s/%s' % (spec_str, member)
        self.spec_str = spec_str

    def __str__(self):
        return self.spec_str

    def __repr__(self):
        return '<GroupSpec %r>' % (self.spec_str)

    @property
    def org(self):
        if self._org is None:
            raise UserError('Organization not given (got %r expected <organization>)' % (self.spec_str,))
        return self._org

    @property
    def group_name(self):
        if self._group_name is None:
            raise UserError('Group name not given (got %r expected <organization>/<group_name>)' % (self.spec_str,))
        return self._group_name

    @property
    def member(self):
        if self._member is None:
            raise UserError('Member name not given (got %r expected <organization>/<group_name>/<member>)' % (self.spec_str,))
        return self._member


def group_spec(spec):
    '''<organization>/<group_name>/<member>'''
    org = spec
    group = member = None
    if '/' in org:
        org, group = org.split('/', 1)
    if group and '/' in group:
        group, member = group.split('/', 1)
    if member and '/' in member:
        raise UserError('Invalid group specification "%s" (unexpected "/" in %s)' % member)

    return GroupSpec(org, group, member, spec)



#  tests for anaconda-client-1.6.5-py35hd900ed1_0 (this is a generated file);
print('===== testing package: anaconda-client-1.6.5-py35hd900ed1_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import binstar_client

print('binstar_client.__version__: %s' % binstar_client.__version__)
assert binstar_client.__version__.startswith('1.6.5')
#  --- run_test.py (end) ---

print('===== anaconda-client-1.6.5-py35hd900ed1_0 OK =====');
print("import: 'binstar_client'")
import binstar_client

print("import: 'binstar_client.scripts.cli'")
import binstar_client.scripts.cli




from __future__ import unicode_literals

from os.path import join, exists
import os
import unittest
import shutil
import tempfile
import subprocess
import sys

from binstar_client.scripts import cli
from binstar_client.tests.fixture import CLITestCase


class Test(CLITestCase):

    def test_write_env(self):
        tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmpdir)

        env = dict(os.environ)
        env[str('BINSTAR_CONFIG_DIR')] = str(tmpdir)

        def main(*args):
            return subprocess.check_output((sys.executable, cli.__file__) + args, env=env)

        main('config', '--set', 'url', 'http://localhost:5000')

        self.assertTrue(exists(join(tmpdir, 'data')))
        self.assertTrue(exists(join(tmpdir, 'data', 'config.yaml')))
        self.assertEqual(open(join(tmpdir, 'data', 'config.yaml')).read(), '''\
url: http://localhost:5000
''')

        output = main('config', '--show-sources')

        self.assertEqual(output.replace(b'\r', b''), u'''\
==> {config} <==
url: http://localhost:5000

'''.format(config=join(tmpdir, 'data', 'config.yaml')).encode('utf-8'))


if __name__ == '__main__':
    unittest.main()



'''
Print the information of the current user
'''
from __future__ import unicode_literals
from binstar_client import errors
from binstar_client.utils import get_server_api
from binstar_client.utils.pprint import pprint_user
import logging

log = logging.getLogger('binstar.whoami')

def main(args):
    aserver_api = get_server_api(args.token, args.site, args.log_level)

    try:
        user = aserver_api.user()
    except errors.Unauthorized as err:
        log.debug(err)
        log.info('Anonymous User')
        return 1

    pprint_user(user)


def add_parser(subparsers):
    subparser = subparsers.add_parser('whoami',
                                      help='Print the information of the current user',
                                      description=__doc__)

    subparser.set_defaults(main=main)
